from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithUserPatchResponse_accessLevels(Parsable):
    """
    Flags indicating the user's access levels in the account.
    """
    # Indicates whether the user is an account administrator for the account. Possible values:- ``true``: The user is an account administrator.- ``false``: The user is not an account administrator.
    account_admin: Optional[bool] = None
    # Indicates whether the user is an executive in the account. Possible values:- ``true``: The user is an executive.- ``false``: The user is not an executive.
    executive: Optional[bool] = None
    # Indicates whether the user is a project administrator for the project. Possible values:- ``true``: The user is a project administrator.- ``false``: The user is not a project administrator.
    project_admin: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithUserPatchResponse_accessLevels:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithUserPatchResponse_accessLevels
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithUserPatchResponse_accessLevels()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "accountAdmin": lambda n : setattr(self, 'account_admin', n.get_bool_value()),
            "executive": lambda n : setattr(self, 'executive', n.get_bool_value()),
            "projectAdmin": lambda n : setattr(self, 'project_admin', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("accountAdmin", self.account_admin)
        writer.write_bool_value("executive", self.executive)
        writer.write_bool_value("projectAdmin", self.project_admin)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .requirements_get_response_spec_sections_submittal_groups_submittal_group_types_submittal_type import RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes_submittalType

@dataclass
class RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes(Parsable):
    # The type of submittal. This corresponds to the *Submittal type* column in the UI.Possible values: ``Test Reports``, ``Shop Drawings``, ``Schedules``, ``Samples``, ``Sample Warranty``, ``Reports``, ``Qualification Data``, ``QUALITY ASSURANCE``, ``Product Data``, ``Performance Data``, ``Mfg. Instructions``, ``Meeting/Conferences``, ``Drawings``, ``Delegated-Design``, ``Certifications``, ``Certificates``, ``Calculations``, ``Attic Stock``, ``Demonstrations``, ``General Warranties``, ``O&M Manuals``, ``Special Warranties``, ``LEED``, ``As-Builts``, ``TESTS AND INSPECTIONS``, ``General``, ``Manufacturers Instructions``, ``Substitutions``, ``Mix Design``, ``Others``
    submittal_type: Optional[RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes_submittalType] = None
    # The number of submittals for the submittal type.
    total: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .requirements_get_response_spec_sections_submittal_groups_submittal_group_types_submittal_type import RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes_submittalType

        from .requirements_get_response_spec_sections_submittal_groups_submittal_group_types_submittal_type import RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes_submittalType

        fields: dict[str, Callable[[Any], None]] = {
            "submittalType": lambda n : setattr(self, 'submittal_type', n.get_enum_value(RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes_submittalType)),
            "total": lambda n : setattr(self, 'total', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("submittalType", self.submittal_type)
        writer.write_int_value("total", self.total)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ValuesBatchUpdatePutResponse_tabularValues_worklogEntries(Parsable):
    # Indicates if the work log row has been deleted.
    deleted: Optional[bool] = None
    # A text description of the work performed.
    description: Optional[str] = None
    # Number of workers.
    headcount: Optional[int] = None
    # Unique identifier for the work log row.
    id: Optional[str] = None
    # Total duration of work performed (in milliseconds).
    timespan: Optional[int] = None
    # A text field indicating the worker's trade.
    trade: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ValuesBatchUpdatePutResponse_tabularValues_worklogEntries:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ValuesBatchUpdatePutResponse_tabularValues_worklogEntries
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ValuesBatchUpdatePutResponse_tabularValues_worklogEntries()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "deleted": lambda n : setattr(self, 'deleted', n.get_bool_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "headcount": lambda n : setattr(self, 'headcount', n.get_int_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "timespan": lambda n : setattr(self, 'timespan', n.get_int_value()),
            "trade": lambda n : setattr(self, 'trade', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("deleted", self.deleted)
        writer.write_str_value("description", self.description)
        writer.write_int_value("headcount", self.headcount)
        writer.write_str_value("id", self.id)
        writer.write_int_value("timespan", self.timespan)
        writer.write_str_value("trade", self.trade)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .item.with_version_set_item_request_builder import WithVersionSetItemRequestBuilder
    from .version_sets_get_response import VersionSetsGetResponse
    from .version_sets_post_request_body import VersionSetsPostRequestBody
    from .version_sets_post_response import VersionSetsPostResponse

class VersionSetsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/sheets/v1/projects/{projectId}/version-sets
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new VersionSetsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/sheets/v1/projects/{projectId}/version-sets{?collectionId*,limit*,offset*,sort*}", path_parameters)
    
    def by_version_set_id(self,version_set_id: str) -> WithVersionSetItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.sheets.v1.projects.item.versionSets.item collection
        param version_set_id: The ID of the version set. To find the version set ID, call `GET version-sets </en/docs/acc/v1/reference/http/sheets-version-sets-GET/>`_.
        Returns: WithVersionSetItemRequestBuilder
        """
        if version_set_id is None:
            raise TypeError("version_set_id cannot be null.")
        from .item.with_version_set_item_request_builder import WithVersionSetItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["versionSetId"] = version_set_id
        return WithVersionSetItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[VersionSetsRequestBuilderGetQueryParameters]] = None) -> Optional[VersionSetsGetResponse]:
        """
        Retrieves a list of version sets.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[VersionSetsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .version_sets_get_response import VersionSetsGetResponse

        return await self.request_adapter.send_async(request_info, VersionSetsGetResponse, None)
    
    async def post(self,body: VersionSetsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[VersionSetsPostResponse]:
        """
        Creates a version set.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[VersionSetsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .version_sets_post_response import VersionSetsPostResponse

        return await self.request_adapter.send_async(request_info, VersionSetsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[VersionSetsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves a list of version sets.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: VersionSetsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a version set.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> VersionSetsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: VersionSetsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return VersionSetsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class VersionSetsRequestBuilderGetQueryParameters():
        """
        Retrieves a list of version sets.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "collection_id":
                return "collectionId"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Filter by sheet collection. If not provided, only results in the ungrouped collection are returned.Possible values:- The UUID of an existing collection: Returns only the results within that collection.- ``*`` Returns results from all collections.
        collection_id: Optional[str] = None

        # The maximum number of results to return per page. The default is ``200``.
        limit: Optional[int] = None

        # The starting point for the results, specified by item number. The default value is ``0``. For example, use ``offset=3`` to start the results from the third item.
        offset: Optional[int] = None

        # Sort the version sets by ``issuanceDate`` or ``name``. You need to add whether to sort in ascending (``asc``) or descending (``desc``) order. For example, ``sort=issuanceDate desc``.
        sort: Optional[str] = None

    
    @dataclass
    class VersionSetsRequestBuilderGetRequestConfiguration(RequestConfiguration[VersionSetsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class VersionSetsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_job_get_response_job import WithJobGetResponse_job
    from .with_job_get_response_status import WithJobGetResponse_status

@dataclass
class WithJobGetResponse(Parsable):
    # If this job tracks the creation of assigned clashes, the IDs of the created issues.
    created_issue_ids: Optional[list[str]] = None
    # The GUIDs that uniquely identify the clash groups associated with the job.
    groups: Optional[list[str]] = None
    # A job.
    job: Optional[WithJobGetResponse_job] = None
    # The GUID that uniquely identifies the job.
    job_id: Optional[UUID] = None
    # The current job status. Possible values: ``Failed``, ``Running``, ``Succeeded``, ``Archived``.
    status: Optional[WithJobGetResponse_status] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithJobGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithJobGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithJobGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_job_get_response_job import WithJobGetResponse_job
        from .with_job_get_response_status import WithJobGetResponse_status

        from .with_job_get_response_job import WithJobGetResponse_job
        from .with_job_get_response_status import WithJobGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "createdIssueIds": lambda n : setattr(self, 'created_issue_ids', n.get_collection_of_primitive_values(str)),
            "groups": lambda n : setattr(self, 'groups', n.get_collection_of_primitive_values(str)),
            "job": lambda n : setattr(self, 'job', n.get_object_value(WithJobGetResponse_job)),
            "jobId": lambda n : setattr(self, 'job_id', n.get_uuid_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithJobGetResponse_status)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("createdIssueIds", self.created_issue_ids)
        writer.write_collection_of_primitive_values("groups", self.groups)
        writer.write_object_value("job", self.job)
        writer.write_uuid_value("jobId", self.job_id)
        writer.write_enum_value("status", self.status)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .revisions_get_response_results_manager_type import RevisionsGetResponse_results_managerType
    from .revisions_get_response_results_steps import RevisionsGetResponse_results_steps
    from .revisions_get_response_results_subcontractor_type import RevisionsGetResponse_results_subcontractorType

@dataclass
class RevisionsGetResponse_results(Parsable):
    # The unique identifier of the submittal item whose revision history is being retrieved. It remains the same across all revisions of the item. Use this ID to track changes and access previous versions. To get the most recent version of a submittal item, call `GET items/:itemId </en/docs/acc/v1/reference/http/submittals-items-itemId-GET/>`_.
    item_id: Optional[UUID] = None
    # The Autodesk ID of the user, company, or role assigned as the manager of the submittal item.In order to get more info about the manager, use:- `GET projects/users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_ to verify the actual name of the user in case the typs is a ``user`` (1).- `GET companies </en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_ to determine the name of the company in case the typs is a ``company`` (2).Note that we do not currently support verifying names of roles.
    manager: Optional[str] = None
    # The date by which the manager is expected to prepare the submittal item for review, formatted as YYYY-MM-DD in UTC (ISO 8601). For example, ``2018-02-15``. This corresponds to the ``mgr-1`` state ``Open (Submitted)``.
    manager_due_date: Optional[str] = None
    # The type of entity assigned as the manager of the submittal item.Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    manager_type: Optional[RevisionsGetResponse_results_managerType] = None
    # The Autodesk ID of the user who published the submittal item.
    published_by: Optional[str] = None
    # The date when the manager closed and distributed the submittal item,  formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    published_date: Optional[datetime.datetime] = None
    # The date and time when the submittal item transitioned from the ``rev`` state (``Open - In Review``) to the ``mgr-2`` state ``(Close and distribute)``, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    received_from_review: Optional[datetime.datetime] = None
    # The date when the subcontractor submitted the submittal item back to the manager, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``. This corresponds to transition to the ``mgr-1`` state.
    received_from_submitter: Optional[datetime.datetime] = None
    # The date and time when the response was added, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``.
    responded_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user that gave the response to the submittal item.
    responded_by: Optional[str] = None
    # The body of the response comment, containing feedback or instructions related to the submittal item.
    response_comment: Optional[str] = None
    # The ID of the response associated with the submittal item, linking to the specific feedback or action taken. For information about the response, call `GET responses </en/docs/acc/v1/reference/http/submittals-responses-GET/>`_.
    response_id: Optional[UUID] = None
    # Not relevant
    reviewer_due_date: Optional[datetime.datetime] = None
    # The version number of the submittal item, representing its revision history.
    revision: Optional[int] = None
    # The date and time when the submittal item was transitioned to the ``rev`` state ``(Open - In review)``, formatted as ISO 8601.
    sent_to_review: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who transitioned the submittal item to the ``rev`` state (``Open - In review``).
    sent_to_review_by: Optional[str] = None
    # The date when the submittal item transitioned to the subcontractor for review, formatted as YYYY-MM-DDTHH:mm:ss.SSSSSSZ (ISO 8601) in UTC. For example, ``2018-02-15T12:09:24.198466Z``. This corresponds to transition to the ``sbc-1`` state.
    sent_to_submitter: Optional[datetime.datetime] = None
    # A list of workflow steps for the submittal item at a specific revision.
    steps: Optional[list[RevisionsGetResponse_results_steps]] = None
    # The Autodesk ID of the user, company, or role assigned as the subcontractor for the submittal item.In order to get more info about the subcontractor, use:- `GET projects/users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_ to verify the actual name of the user in case the typs is a ``user`` (1).- `GET companies </en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_ to determine the name of the company in case the typs is a ``company`` (2).Note that we do not currently support verifying names of roles.
    subcontractor: Optional[str] = None
    # The type of manager.Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    subcontractor_type: Optional[RevisionsGetResponse_results_subcontractorType] = None
    # The Autodesk ID of the user who transitioned the submittal item to the manager.
    submitted_by: Optional[str] = None
    # The date when the subcontractor is expected to submit the submittal to the manager, in the following format: YYYY-MM-DD in UTC (ISO 8601). For example, ``2017-02-15``. This corresponds to the ``sbc-1`` state ``Waiting for submission``.
    submitter_due_date: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RevisionsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RevisionsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RevisionsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .revisions_get_response_results_manager_type import RevisionsGetResponse_results_managerType
        from .revisions_get_response_results_steps import RevisionsGetResponse_results_steps
        from .revisions_get_response_results_subcontractor_type import RevisionsGetResponse_results_subcontractorType

        from .revisions_get_response_results_manager_type import RevisionsGetResponse_results_managerType
        from .revisions_get_response_results_steps import RevisionsGetResponse_results_steps
        from .revisions_get_response_results_subcontractor_type import RevisionsGetResponse_results_subcontractorType

        fields: dict[str, Callable[[Any], None]] = {
            "itemId": lambda n : setattr(self, 'item_id', n.get_uuid_value()),
            "manager": lambda n : setattr(self, 'manager', n.get_str_value()),
            "managerDueDate": lambda n : setattr(self, 'manager_due_date', n.get_str_value()),
            "managerType": lambda n : setattr(self, 'manager_type', n.get_enum_value(RevisionsGetResponse_results_managerType)),
            "publishedBy": lambda n : setattr(self, 'published_by', n.get_str_value()),
            "publishedDate": lambda n : setattr(self, 'published_date', n.get_datetime_value()),
            "receivedFromReview": lambda n : setattr(self, 'received_from_review', n.get_datetime_value()),
            "receivedFromSubmitter": lambda n : setattr(self, 'received_from_submitter', n.get_datetime_value()),
            "respondedAt": lambda n : setattr(self, 'responded_at', n.get_datetime_value()),
            "respondedBy": lambda n : setattr(self, 'responded_by', n.get_str_value()),
            "responseComment": lambda n : setattr(self, 'response_comment', n.get_str_value()),
            "responseId": lambda n : setattr(self, 'response_id', n.get_uuid_value()),
            "reviewerDueDate": lambda n : setattr(self, 'reviewer_due_date', n.get_datetime_value()),
            "revision": lambda n : setattr(self, 'revision', n.get_int_value()),
            "sentToReview": lambda n : setattr(self, 'sent_to_review', n.get_datetime_value()),
            "sentToReviewBy": lambda n : setattr(self, 'sent_to_review_by', n.get_str_value()),
            "sentToSubmitter": lambda n : setattr(self, 'sent_to_submitter', n.get_datetime_value()),
            "steps": lambda n : setattr(self, 'steps', n.get_collection_of_object_values(RevisionsGetResponse_results_steps)),
            "subcontractor": lambda n : setattr(self, 'subcontractor', n.get_str_value()),
            "subcontractorType": lambda n : setattr(self, 'subcontractor_type', n.get_enum_value(RevisionsGetResponse_results_subcontractorType)),
            "submittedBy": lambda n : setattr(self, 'submitted_by', n.get_str_value()),
            "submitterDueDate": lambda n : setattr(self, 'submitter_due_date', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("itemId", self.item_id)
        writer.write_str_value("manager", self.manager)
        writer.write_str_value("managerDueDate", self.manager_due_date)
        writer.write_enum_value("managerType", self.manager_type)
        writer.write_str_value("publishedBy", self.published_by)
        writer.write_datetime_value("publishedDate", self.published_date)
        writer.write_datetime_value("receivedFromReview", self.received_from_review)
        writer.write_datetime_value("receivedFromSubmitter", self.received_from_submitter)
        writer.write_datetime_value("respondedAt", self.responded_at)
        writer.write_str_value("respondedBy", self.responded_by)
        writer.write_str_value("responseComment", self.response_comment)
        writer.write_uuid_value("responseId", self.response_id)
        writer.write_datetime_value("reviewerDueDate", self.reviewer_due_date)
        writer.write_int_value("revision", self.revision)
        writer.write_datetime_value("sentToReview", self.sent_to_review)
        writer.write_str_value("sentToReviewBy", self.sent_to_review_by)
        writer.write_datetime_value("sentToSubmitter", self.sent_to_submitter)
        writer.write_collection_of_object_values("steps", self.steps)
        writer.write_str_value("subcontractor", self.subcontractor)
        writer.write_enum_value("subcontractorType", self.subcontractor_type)
        writer.write_str_value("submittedBy", self.submitted_by)
        writer.write_str_value("submitterDueDate", self.submitter_due_date)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .me_get_response_issues import MeGetResponse_issues

@dataclass
class MeGetResponse(Parsable):
    # Not relevant
    can_manage_templates: Optional[bool] = None
    # The user's Autodesk ID.
    id: Optional[str] = None
    # States whether the current logged in user is a system admin.
    is_project_admin: Optional[bool] = None
    # The issues property
    issues: Optional[list[MeGetResponse_issues]] = None
    # The permission level of the user. Each permission level corresponds to a combination of values in the response. For example, a combination of ``read`` and ``create`` in the response, corresponds to a *Full visibility* permission level.Note that if a user with *View and assign issues for their company* permissions attempts to assign a user from a another company to the issue, it will return an error. In addition, the user can both create and view issues for their own company. You can also verify a user's assignment permissions by checking the ``permittedActions`` or ``permissionLevels`` attributes.- *Edit, view, and assign* This permission level is split into two sub-levels:- *View and assign to their company* (previously known as  *Create for my company*) : ``create`` and the ``permittedActions`` array must include ``assign-same-company``- *View issues for their company. Assign issues to anyone.* : ``create`` and the ``permittedActions`` array must include ``assign-all``- *Full visibility* (previously known as *Create for other companies*): ``create``, ``read``- *Manage issues*: ``create``, ``read``, ``write``Possible values: ``create``, ``read``, ``write``.For more details about the permission levels, see `Issues Permissions <https://help.autodesk.com/view/BUILD/ENU/?guid=Issues_Permissions>`_.
    permission_levels: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MeGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MeGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MeGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .me_get_response_issues import MeGetResponse_issues

        from .me_get_response_issues import MeGetResponse_issues

        fields: dict[str, Callable[[Any], None]] = {
            "canManageTemplates": lambda n : setattr(self, 'can_manage_templates', n.get_bool_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "isProjectAdmin": lambda n : setattr(self, 'is_project_admin', n.get_bool_value()),
            "issues": lambda n : setattr(self, 'issues', n.get_collection_of_object_values(MeGetResponse_issues)),
            "permissionLevels": lambda n : setattr(self, 'permission_levels', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("canManageTemplates", self.can_manage_templates)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("isProjectAdmin", self.is_project_admin)
        writer.write_collection_of_object_values("issues", self.issues)
        writer.write_collection_of_primitive_values("permissionLevels", self.permission_levels)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .exports_post_request_body_options_issue_markups import ExportsPostRequestBody_options_issueMarkups
    from .exports_post_request_body_options_photo_markups import ExportsPostRequestBody_options_photoMarkups
    from .exports_post_request_body_options_standard_markups import ExportsPostRequestBody_options_standardMarkups

@dataclass
class ExportsPostRequestBody_options(Parsable):
    """
    The criteria for the markups and links that should be included with the exported sheets.Note that unpublished markups are only visible to their creator.
    """
    # The list of types of issue markups to export.
    issue_markups: Optional[ExportsPostRequestBody_options_issueMarkups] = None
    # The name of the file that will be generated by the export process. Note that this name should not include the file extension. The maximum character limit for this field is 255 characters.
    output_file_name: Optional[str] = None
    # The list of types of photo markups to export.
    photo_markups: Optional[ExportsPostRequestBody_options_photoMarkups] = None
    # The list of types of standard markups and their associated links that you want to include in the export.By default markups will be exported and hyperlinks will not be exported.
    standard_markups: Optional[ExportsPostRequestBody_options_standardMarkups] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ExportsPostRequestBody_options:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ExportsPostRequestBody_options
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ExportsPostRequestBody_options()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .exports_post_request_body_options_issue_markups import ExportsPostRequestBody_options_issueMarkups
        from .exports_post_request_body_options_photo_markups import ExportsPostRequestBody_options_photoMarkups
        from .exports_post_request_body_options_standard_markups import ExportsPostRequestBody_options_standardMarkups

        from .exports_post_request_body_options_issue_markups import ExportsPostRequestBody_options_issueMarkups
        from .exports_post_request_body_options_photo_markups import ExportsPostRequestBody_options_photoMarkups
        from .exports_post_request_body_options_standard_markups import ExportsPostRequestBody_options_standardMarkups

        fields: dict[str, Callable[[Any], None]] = {
            "issueMarkups": lambda n : setattr(self, 'issue_markups', n.get_object_value(ExportsPostRequestBody_options_issueMarkups)),
            "outputFileName": lambda n : setattr(self, 'output_file_name', n.get_str_value()),
            "photoMarkups": lambda n : setattr(self, 'photo_markups', n.get_object_value(ExportsPostRequestBody_options_photoMarkups)),
            "standardMarkups": lambda n : setattr(self, 'standard_markups', n.get_object_value(ExportsPostRequestBody_options_standardMarkups)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("issueMarkups", self.issue_markups)
        writer.write_str_value("outputFileName", self.output_file_name)
        writer.write_object_value("photoMarkups", self.photo_markups)
        writer.write_object_value("standardMarkups", self.standard_markups)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .companies_get_response import CompaniesGetResponse
    from .get_filter_text_match_query_parameter_type import GetFilterTextMatchQueryParameterType

class CompaniesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/admin/v1/accounts/{accountId}/companies
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new CompaniesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/admin/v1/accounts/{accountId}/companies{?fields*,filterTextMatch*,filter%5BerpId%5D*,filter%5Bname%5D*,filter%5BtaxId%5D*,filter%5Btrade%5D*,filter%5BupdatedAt%5D*,limit*,offset*,orFilters*,sort*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[CompaniesRequestBuilderGetQueryParameters]] = None) -> Optional[CompaniesGetResponse]:
        """
        Returns a list of companies in an account.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CompaniesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .companies_get_response import CompaniesGetResponse

        return await self.request_adapter.send_async(request_info, CompaniesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[CompaniesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Returns a list of companies in an account.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> CompaniesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: CompaniesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return CompaniesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class CompaniesRequestBuilderGetQueryParameters():
        """
        Returns a list of companies in an account.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filtererp_id":
                return "filter%5BerpId%5D"
            if original_name == "filtername":
                return "filter%5Bname%5D"
            if original_name == "filtertax_id":
                return "filter%5BtaxId%5D"
            if original_name == "filtertrade":
                return "filter%5Btrade%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "filter_text_match":
                return "filterTextMatch"
            if original_name == "or_filters":
                return "orFilters"
            if original_name == "fields":
                return "fields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # List of fields to return in the response.  Defaults to all fields.  Valid list of fields are `accountId`, `name`, `trade`, `addresses`, `websiteUrl`, `description`, `erpId`, `taxId`, `imageUrl`, `status`, `createdAt`, `updatedAt`, `projectSize`, `userSize` and `originalName`.
        fields: Optional[list[str]] = None

        # Specifies how text-based filters should match values in supported fields.This parameter can be used in any endpoint that supports text-based filtering (e.g., ``filter[name]``, ``filter[jobNumber]``, ``filter[companyName]``, etc.).Possible values:``contains`` (default) – Matches if the field contains the specified text anywhere``startsWith`` – Matches if the field starts with the specified text``endsWith`` – Matches if the field ends with the specified text``equals`` – Matches only if the field exactly matches the specified textMatching is case-insensitive.Wildcards and regular expressions are not supported.
        filter_text_match: Optional[GetFilterTextMatchQueryParameterType] = None

        # Filter companies by ERP Id.  Can be a partial match based on the value of `filterTextMatch` provided.Max length: 255
        filtererp_id: Optional[str] = None

        # Filter companies by name.  Can be a partial match based on the value of `filterTextMatch` provided.Max length: 255
        filtername: Optional[str] = None

        # Filter companies by tax Id.  Can be a partial match based on the value of `filterTextMatch` provided.Max length: 255
        filtertax_id: Optional[str] = None

        # Filter companies by trade.  Can be a partial match based on the value of `filterTextMatch` provided.Max length: 255
        filtertrade: Optional[str] = None

        # Filter companies by updated at date range. The range must be specified with dates in an ISO-8601 format with time required.  The start and end dates of the range should be separated by `..`  One of the dates in the range may be omitted. For example, to get everything on or before June 1, 2019 the range would be `..2019-06-01T23:59:59.999Z`. To get everything after June 1, 2019 the range would be `2019-06-01T00:00:00.000Z..`.Max length: 100
        filterupdated_at: Optional[str] = None

        # The maximum number of records to return in the response.Default: ``20``Minimum: ``1``Maximum: ``200`` (If a larger value is provided, only 200 records are returned)
        limit: Optional[int] = None

        # The index of the first record to return.Used for pagination in combination with the ``limit`` parameter.Example: ``limit=20`` and ``offset=40`` returns records 41–60.
        offset: Optional[int] = None

        # List of filtered fields to apply an ``or`` operator. Valid list of fields are `erpId`, `name`, `taxId`, `trade`, `updatedAt`.
        or_filters: Optional[list[str]] = None

        # The list of fields to sort by.  When multiple fields are listed the later property is used to sort the resources where the previous fields have the same value. Each property can be followed by a direction modifier of either `asc` (ascending) or `desc` (descending).  If no direction is specified then `asc` is assumed.  Valid fields for sorting are `name`, `trade`, `erpId`, `taxId`, `status`, `createdAt`, `updatedAt`, `projectSize` and `userSize`.  Default sort is `name`.
        sort: Optional[list[str]] = None

    
    @dataclass
    class CompaniesRequestBuilderGetRequestConfiguration(RequestConfiguration[CompaniesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .mappings_get_response_results_submittals_role import MappingsGetResponse_results_submittalsRole
    from .mappings_get_response_results_user_type import MappingsGetResponse_results_userType

@dataclass
class MappingsGetResponse_results(Parsable):
    # The Autodesk ID of the user (``autodeskId``), role (``memberGroupId``), or company (``memberGroupId``).To find details about users, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_, to find details about companies, call `GET companies <en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_. Note that we do not currently support finding details about roles for a project.
    autodesk_id: Optional[str] = None
    # When the record was created.
    created_at: Optional[datetime.datetime] = None
    # Autodesk ID of the user who has created the project.
    created_by: Optional[str] = None
    # The unique identifier (UUID) of the user-role mapping record.
    id: Optional[UUID] = None
    # Not relevant
    oxygen_id: Optional[str] = None
    # The role of the user in Submittals. Possible values: ``1`` (manager).
    submittals_role: Optional[MappingsGetResponse_results_submittalsRole] = None
    # When the record was last updated.
    updated_at: Optional[datetime.datetime] = None
    # Autodesk ID of the last user who updated the record.
    updated_by: Optional[str] = None
    # The ID for the type of user in the record. Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    user_type: Optional[MappingsGetResponse_results_userType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MappingsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MappingsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MappingsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .mappings_get_response_results_submittals_role import MappingsGetResponse_results_submittalsRole
        from .mappings_get_response_results_user_type import MappingsGetResponse_results_userType

        from .mappings_get_response_results_submittals_role import MappingsGetResponse_results_submittalsRole
        from .mappings_get_response_results_user_type import MappingsGetResponse_results_userType

        fields: dict[str, Callable[[Any], None]] = {
            "autodeskId": lambda n : setattr(self, 'autodesk_id', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "oxygenId": lambda n : setattr(self, 'oxygen_id', n.get_str_value()),
            "submittalsRole": lambda n : setattr(self, 'submittals_role', n.get_enum_value(MappingsGetResponse_results_submittalsRole)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "userType": lambda n : setattr(self, 'user_type', n.get_enum_value(MappingsGetResponse_results_userType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("autodeskId", self.autodesk_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("oxygenId", self.oxygen_id)
        writer.write_enum_value("submittalsRole", self.submittals_role)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_enum_value("userType", self.user_type)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_model_set_get_response_folders import WithModelSetGetResponse_folders
    from .with_model_set_get_response_model_set_type import WithModelSetGetResponse_modelSetType

@dataclass
class WithModelSetGetResponse(Parsable):
    # The GUID that uniquely identifies the container.
    container_id: Optional[UUID] = None
    # The unique identifier of the user who created the model set.
    created_by: Optional[str] = None
    # The date and time that the model set was created.
    created_time: Optional[datetime.datetime] = None
    # A textual description of the model set. Min length: 1 Max length: 1024.
    description: Optional[str] = None
    # The ModelSetFolders that determine the documents included in the model set.
    folders: Optional[list[WithModelSetGetResponse_folders]] = None
    # If set to true, a model set version is not automatically created following changes to documents within the model set.
    is_disabled: Optional[bool] = None
    # The GUID that uniquely identifies the model set.
    model_set_id: Optional[UUID] = None
    # The type of content this model set tracks. Possible values: ``Plans``, ``ProjectFiles``.
    model_set_type: Optional[WithModelSetGetResponse_modelSetType] = None
    # The name of the model set. This name must be unique within the specified container. Min length: 1 Max length: 64.
    name: Optional[str] = None
    # The version number of the most recent version of the model set.
    tip_version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithModelSetGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithModelSetGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithModelSetGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_model_set_get_response_folders import WithModelSetGetResponse_folders
        from .with_model_set_get_response_model_set_type import WithModelSetGetResponse_modelSetType

        from .with_model_set_get_response_folders import WithModelSetGetResponse_folders
        from .with_model_set_get_response_model_set_type import WithModelSetGetResponse_modelSetType

        fields: dict[str, Callable[[Any], None]] = {
            "containerId": lambda n : setattr(self, 'container_id', n.get_uuid_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdTime": lambda n : setattr(self, 'created_time', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "folders": lambda n : setattr(self, 'folders', n.get_collection_of_object_values(WithModelSetGetResponse_folders)),
            "isDisabled": lambda n : setattr(self, 'is_disabled', n.get_bool_value()),
            "modelSetId": lambda n : setattr(self, 'model_set_id', n.get_uuid_value()),
            "modelSetType": lambda n : setattr(self, 'model_set_type', n.get_enum_value(WithModelSetGetResponse_modelSetType)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "tipVersion": lambda n : setattr(self, 'tip_version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("containerId", self.container_id)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("createdTime", self.created_time)
        writer.write_str_value("description", self.description)
        writer.write_collection_of_object_values("folders", self.folders)
        writer.write_bool_value("isDisabled", self.is_disabled)
        writer.write_uuid_value("modelSetId", self.model_set_id)
        writer.write_enum_value("modelSetType", self.model_set_type)
        writer.write_str_value("name", self.name)
        writer.write_int_value("tipVersion", self.tip_version)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .custom_attribute_definitions.custom_attribute_definitions_request_builder import CustomAttributeDefinitionsRequestBuilder
    from .permissions.permissions_request_builder import PermissionsRequestBuilder
    from .permissions_batch_create.permissions_batch_create_request_builder import PermissionsBatchCreateRequestBuilder
    from .permissions_batch_delete.permissions_batch_delete_request_builder import PermissionsBatchDeleteRequestBuilder
    from .permissions_batch_update.permissions_batch_update_request_builder import PermissionsBatchUpdateRequestBuilder

class WithFolder_ItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/docs/v1/projects/{project-id}/folders/{folder_id}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithFolder_ItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/docs/v1/projects/{project%2Did}/folders/{folder_id}", path_parameters)
    
    @property
    def custom_attribute_definitions(self) -> CustomAttributeDefinitionsRequestBuilder:
        """
        The customAttributeDefinitions property
        """
        from .custom_attribute_definitions.custom_attribute_definitions_request_builder import CustomAttributeDefinitionsRequestBuilder

        return CustomAttributeDefinitionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def permissions(self) -> PermissionsRequestBuilder:
        """
        The permissions property
        """
        from .permissions.permissions_request_builder import PermissionsRequestBuilder

        return PermissionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def permissions_batch_create(self) -> PermissionsBatchCreateRequestBuilder:
        """
        The permissionsBatchCreate property
        """
        from .permissions_batch_create.permissions_batch_create_request_builder import PermissionsBatchCreateRequestBuilder

        return PermissionsBatchCreateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def permissions_batch_delete(self) -> PermissionsBatchDeleteRequestBuilder:
        """
        The permissionsBatchDelete property
        """
        from .permissions_batch_delete.permissions_batch_delete_request_builder import PermissionsBatchDeleteRequestBuilder

        return PermissionsBatchDeleteRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def permissions_batch_update(self) -> PermissionsBatchUpdateRequestBuilder:
        """
        The permissionsBatchUpdate property
        """
        from .permissions_batch_update.permissions_batch_update_request_builder import PermissionsBatchUpdateRequestBuilder

        return PermissionsBatchUpdateRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .contracts_post_response_additional_contacts import ContractsPostResponse_additionalContacts
    from .contracts_post_response_recipients import ContractsPostResponse_recipients

@dataclass
class ContractsPostResponse(Parsable):
    # Not relevant
    additional_contacts: Optional[list[ContractsPostResponse_additionalContacts]] = None
    # The date and time of contract approval, in ISO 8601 format.
    approved_at: Optional[str] = None
    # The date and time of the contract award, in ISO 8601 format.
    awarded_at: Optional[str] = None
    # The code of the contract.Max length: 255
    code: Optional[str] = None
    # Not relevant
    company_e_r_p_id: Optional[str] = None
    # The ID of a company managed by a BIM 360 Admin. Detailed company information can be retrieved by calling `GET projects/:project_id/companies <en/docs/bim360/v1/reference/http/projects-:project_id-companies-GET/>`_ and locating the `member_group_id` in the response.
    company_id: Optional[str] = None
    # Not relevant
    company_tax_id: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    company_uid: Optional[str] = None
    # The completed work retention percentage of the contract amount.
    completed_work_retention_percent: Optional[float] = None
    # The unique identifier of the default contact for the supplier. This ID is generated and managed by the BIM 360 Admin service. The ``contactId`` can be used to query user information such as ``email`` and ``company``.
    contact_id: Optional[str] = None
    # The code of the currency specified for the contract if it's awarded in a foreign currency.
    currency: Optional[str] = None
    # The detailed description of a contract.Max length: 2048
    description: Optional[str] = None
    # The final exchange rate for the specified ``currency``, applied as a multiplier of the contract's base currency. For example, 1 base currency = 0.7455 foreign currency.
    exchange_rate: Optional[float] = None
    # The date and time of contract execution, in ISO 8601 format.
    executed_at: Optional[str] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The forecast exchange rate. Default value: ``null``.
    forecast_exchange_rate: Optional[float] = None
    # The last time that the forecast exchange rate was updated, in ISO 8601 format.
    forecast_exchange_rate_updated_at: Optional[datetime.datetime] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # Not relevant
    internal_id: Optional[str] = None
    # Not relevant
    internal_system: Optional[str] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # The ID of the main contract with which this item is associated.
    main_contract_id: Optional[str] = None
    # The materials retention percentage of the contract amount.
    materials_retention_percent: Optional[float] = None
    # The name of the contract.Max length: 1024
    name: Optional[str] = None
    # The date and time of job completion by the supplier, in ISO 8601 format.
    offsite_at: Optional[str] = None
    # The date and time of the supplier's arrival on-site, in ISO 8601 format.
    onsite_at: Optional[str] = None
    # The user responsible for the purchase. This is the ID of a user managed by BIM 360 Admin.
    owner_id: Optional[str] = None
    # The date and time of purchase, in ISO 8601 format. This is designed for Purchase Order contracts.
    procured_at: Optional[str] = None
    # Not relevant
    recipients: Optional[list[ContractsPostResponse_recipients]] = None
    # The date and time of the supplier's response, in ISO 8601 format.
    responded_at: Optional[str] = None
    # The date and time by which the supplier response is due, in ISO 8601 format.
    response_due: Optional[str] = None
    # The maximum percentage of the total contract amount which can be used as the retention amount.
    retention_cap: Optional[float] = None
    # The date and time of the signed contract return, in ISO 8601 format.
    returned_at: Optional[str] = None
    # The date and time of contract transmission to the supplier, in ISO 8601 format.
    sent_at: Optional[str] = None
    # The user who signed the contract. This is the ID of a user managed by BIM 360 Admin.
    signed_by: Optional[str] = None
    # The URN of the source entity from which the contract is created.
    source: Optional[str] = None
    # The status of this contract.Possible values: ``draft``, ``pending``, ``submitted``, ``revise``, ``sent``, ``signed``, ``executed``, ``closed``, ``inReview``
    status: Optional[str] = None
    # The date and time of the last status change, in ISO 8601 format.
    status_changed_at: Optional[datetime.datetime] = None
    # The subStatus is used when the contract is in an ``executed`` state and may have additional statuses under it. For example, if a contract is ``executed`` but requires the supplier to provide additional information, the status will be executed and the subStatus will be pending. In other cases, subStatus is always ``null``. Possible values: ``null``, ``pending``, ``submitted``, ``revise``. Default value: ``null``.
    sub_status: Optional[str] = None
    # The type of contract. For example, ``consultant`` or ``purchase order``. The ``type`` can be customized by the project admin.
    type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ContractsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ContractsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ContractsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .contracts_post_response_additional_contacts import ContractsPostResponse_additionalContacts
        from .contracts_post_response_recipients import ContractsPostResponse_recipients

        from .contracts_post_response_additional_contacts import ContractsPostResponse_additionalContacts
        from .contracts_post_response_recipients import ContractsPostResponse_recipients

        fields: dict[str, Callable[[Any], None]] = {
            "additionalContacts": lambda n : setattr(self, 'additional_contacts', n.get_collection_of_object_values(ContractsPostResponse_additionalContacts)),
            "approvedAt": lambda n : setattr(self, 'approved_at', n.get_str_value()),
            "awardedAt": lambda n : setattr(self, 'awarded_at', n.get_str_value()),
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "companyERPId": lambda n : setattr(self, 'company_e_r_p_id', n.get_str_value()),
            "companyId": lambda n : setattr(self, 'company_id', n.get_str_value()),
            "companyTaxId": lambda n : setattr(self, 'company_tax_id', n.get_str_value()),
            "companyUid": lambda n : setattr(self, 'company_uid', n.get_str_value()),
            "completedWorkRetentionPercent": lambda n : setattr(self, 'completed_work_retention_percent', n.get_float_value()),
            "contactId": lambda n : setattr(self, 'contact_id', n.get_str_value()),
            "currency": lambda n : setattr(self, 'currency', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "exchangeRate": lambda n : setattr(self, 'exchange_rate', n.get_float_value()),
            "executedAt": lambda n : setattr(self, 'executed_at', n.get_str_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "forecastExchangeRate": lambda n : setattr(self, 'forecast_exchange_rate', n.get_float_value()),
            "forecastExchangeRateUpdatedAt": lambda n : setattr(self, 'forecast_exchange_rate_updated_at', n.get_datetime_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "internalId": lambda n : setattr(self, 'internal_id', n.get_str_value()),
            "internalSystem": lambda n : setattr(self, 'internal_system', n.get_str_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "mainContractId": lambda n : setattr(self, 'main_contract_id', n.get_str_value()),
            "materialsRetentionPercent": lambda n : setattr(self, 'materials_retention_percent', n.get_float_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "offsiteAt": lambda n : setattr(self, 'offsite_at', n.get_str_value()),
            "onsiteAt": lambda n : setattr(self, 'onsite_at', n.get_str_value()),
            "ownerId": lambda n : setattr(self, 'owner_id', n.get_str_value()),
            "procuredAt": lambda n : setattr(self, 'procured_at', n.get_str_value()),
            "recipients": lambda n : setattr(self, 'recipients', n.get_collection_of_object_values(ContractsPostResponse_recipients)),
            "respondedAt": lambda n : setattr(self, 'responded_at', n.get_str_value()),
            "responseDue": lambda n : setattr(self, 'response_due', n.get_str_value()),
            "retentionCap": lambda n : setattr(self, 'retention_cap', n.get_float_value()),
            "returnedAt": lambda n : setattr(self, 'returned_at', n.get_str_value()),
            "sentAt": lambda n : setattr(self, 'sent_at', n.get_str_value()),
            "signedBy": lambda n : setattr(self, 'signed_by', n.get_str_value()),
            "source": lambda n : setattr(self, 'source', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "statusChangedAt": lambda n : setattr(self, 'status_changed_at', n.get_datetime_value()),
            "subStatus": lambda n : setattr(self, 'sub_status', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("additionalContacts", self.additional_contacts)
        writer.write_str_value("approvedAt", self.approved_at)
        writer.write_str_value("awardedAt", self.awarded_at)
        writer.write_str_value("code", self.code)
        writer.write_str_value("companyERPId", self.company_e_r_p_id)
        writer.write_str_value("companyId", self.company_id)
        writer.write_str_value("companyTaxId", self.company_tax_id)
        writer.write_str_value("companyUid", self.company_uid)
        writer.write_float_value("completedWorkRetentionPercent", self.completed_work_retention_percent)
        writer.write_str_value("contactId", self.contact_id)
        writer.write_str_value("currency", self.currency)
        writer.write_str_value("description", self.description)
        writer.write_float_value("exchangeRate", self.exchange_rate)
        writer.write_str_value("executedAt", self.executed_at)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_float_value("forecastExchangeRate", self.forecast_exchange_rate)
        writer.write_datetime_value("forecastExchangeRateUpdatedAt", self.forecast_exchange_rate_updated_at)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_str_value("internalId", self.internal_id)
        writer.write_str_value("internalSystem", self.internal_system)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_str_value("mainContractId", self.main_contract_id)
        writer.write_float_value("materialsRetentionPercent", self.materials_retention_percent)
        writer.write_str_value("name", self.name)
        writer.write_str_value("offsiteAt", self.offsite_at)
        writer.write_str_value("onsiteAt", self.onsite_at)
        writer.write_str_value("ownerId", self.owner_id)
        writer.write_str_value("procuredAt", self.procured_at)
        writer.write_collection_of_object_values("recipients", self.recipients)
        writer.write_str_value("respondedAt", self.responded_at)
        writer.write_str_value("responseDue", self.response_due)
        writer.write_float_value("retentionCap", self.retention_cap)
        writer.write_str_value("returnedAt", self.returned_at)
        writer.write_str_value("sentAt", self.sent_at)
        writer.write_str_value("signedBy", self.signed_by)
        writer.write_str_value("source", self.source)
        writer.write_str_value("status", self.status)
        writer.write_datetime_value("statusChangedAt", self.status_changed_at)
        writer.write_str_value("subStatus", self.sub_status)
        writer.write_str_value("type", self.type)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .attachments_post_response_association_type import AttachmentsPostResponse_associationType
    from .attachments_post_response_type import AttachmentsPostResponse_type

@dataclass
class AttachmentsPostResponse(Parsable):
    # The object ID of the item associated with the actions, such as a budget, contract, or cost item.
    association_id: Optional[UUID] = None
    # The type of item to which it is associated.Possible values: ``Budget``, ``Contract``, ``ScheduleOfValue``, ``FormInstance``, ``CostItem``, ``Payment``, ``MainContract``, ``BudgetPayment``, ``Expense``, ``CostPayment``, ``ExpenseItem``, ``PaymentItem``, ``OCO``, ``RCO``, ``SCO``, ``PCO``, ``RFQ``, ``DistributionItem``, ``BudgetTransfer``, ``Fee``
    association_type: Optional[AttachmentsPostResponse_associationType] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The unique identifier (UUID) of the folder named ``attachment-folder`` where the attachment is stored. This ID is automatically retrieved by the backend and returned in the response.
    folder_id: Optional[UUID] = None
    # The unique auto-generated identifier of the attachment.
    id: Optional[UUID] = None
    # The name of the attachment.Max length: 1024
    name: Optional[str] = None
    # Not relevant
    pdf_oss_urn: Optional[str] = None
    # Not relevant
    pdf_urn: Optional[str] = None
    # Determines whether an existing attachment with a matching name should be replaced.
    replace_if_exists: Optional[bool] = None
    # The status of the attachment.Max length: 1024
    status: Optional[str] = None
    # The type of attachment.Possible values:``Upload``: is a locally uploaded file.``DocsFile`` is a file referenced from BIM 360 Docs.``Reference`` is a file referenced form report.``Document`` is for document generation.Max length: 64
    type: Optional[AttachmentsPostResponse_type] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    # The object version URN of the attachment in the Autodesk Data Management service.
    urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AttachmentsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AttachmentsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AttachmentsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .attachments_post_response_association_type import AttachmentsPostResponse_associationType
        from .attachments_post_response_type import AttachmentsPostResponse_type

        from .attachments_post_response_association_type import AttachmentsPostResponse_associationType
        from .attachments_post_response_type import AttachmentsPostResponse_type

        fields: dict[str, Callable[[Any], None]] = {
            "associationId": lambda n : setattr(self, 'association_id', n.get_uuid_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_enum_value(AttachmentsPostResponse_associationType)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "folderId": lambda n : setattr(self, 'folder_id', n.get_uuid_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "pdfOssUrn": lambda n : setattr(self, 'pdf_oss_urn', n.get_str_value()),
            "pdfUrn": lambda n : setattr(self, 'pdf_urn', n.get_str_value()),
            "replaceIfExists": lambda n : setattr(self, 'replace_if_exists', n.get_bool_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(AttachmentsPostResponse_type)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("associationId", self.association_id)
        writer.write_enum_value("associationType", self.association_type)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_uuid_value("folderId", self.folder_id)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("pdfOssUrn", self.pdf_oss_urn)
        writer.write_str_value("pdfUrn", self.pdf_urn)
        writer.write_bool_value("replaceIfExists", self.replace_if_exists)
        writer.write_str_value("status", self.status)
        writer.write_enum_value("type", self.type)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("urn", self.urn)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_version_item_request_builder import WithVersionItemRequestBuilder

class PublishedVersionsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/rcm/v1/projects/{projectId}/published-versions
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new PublishedVersionsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/rcm/v1/projects/{projectId}/published-versions", path_parameters)
    
    def by_version_id(self,version_id: str) -> WithVersionItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.rcm.v1.projects.item.publishedVersions.item collection
        param version_id: The URL-encoded version ID (URN) of the published Revit model version for which you want to retrieve linked files.When a Cloud Workshared Revit model is published in BIM 360 or the Autodesk Construction Cloud (ACC), a new version ID is created for each published Revit model version.Every time a model is updated and published, it receives a new unique version ID.You must provide a version ID to retrieve the linked files for that specific published Revit model version.To retrieve the latest (``tip``) version ID, use `GET projects/:project_id/folders/:folder_id/contents </en/docs/data/v2/reference/http/projects-project_id-folders-folder_id-contents-GET>`_. To filter out non-cloud workshared Revit files, use the ``items:autodesk.bim360:C4RModel`` filter .To retrieve a specific past version, use `GET projects/:project_id/items/:item_id/versions </en/docs/data/v2/reference/http/projects-project_id-items-item_id-versions-GET>`_.For more details about retrieving the version ID, see the `Retrieve Signed URLs for Linked RVT Files </en/docs/acc/v1/tutorials/files/rcm-linked-files>`_ tutorial.
        Returns: WithVersionItemRequestBuilder
        """
        if version_id is None:
            raise TypeError("version_id cannot be null.")
        from .item.with_version_item_request_builder import WithVersionItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["versionId"] = version_id
        return WithVersionItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_e_u_required_attributes_values import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes_values

@dataclass
class RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes(Parsable):
    # The name of the RFI attribute that must be provided when updating or creating an RFI in the specified status.
    name: Optional[str] = None
    # The list of allowed values for the required attribute.
    values: Optional[list[RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes_values]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_e_u_required_attributes_values import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes_values

        from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_e_u_required_attributes_values import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes_values

        fields: dict[str, Callable[[Any], None]] = {
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "values": lambda n : setattr(self, 'values', n.get_collection_of_object_values(RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfEU_requiredAttributes_values)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("name", self.name)
        writer.write_collection_of_object_values("values", self.values)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class StatusStepSetsPostRequestBody_values(Parsable):
    # The color of the status as the status appears in the Assets UI.This field is not restricted as it is primarily a tool for the Assets UI to use.However, there are only certain colors that the Autodesk Construction Cloud Assets UI knows how tooperate with. Understood color values are: ``adsk-black``, ``adsk-white``, ``adsk-charcoal-900``,``adsk-charcoal-800``, ``adsk-charcoal-700``, ``adsk-charcoal-600``, ``adsk-charcoal-500``,``adsk-charcoal-400``, ``adsk-charcoal-300``, ``adsk-charcoal-200``, ``adsk-charcoal-100``,``adsk-charcoal-050``, ``adsk-blue-700``, ``adsk-blue-500``, ``adsk-blue-300``, ``adsk-blue-100``,``adsk-red-700``, ``adsk-red-500``, ``adsk-red-300``, ``adsk-green-700``, ``adsk-green-500``,``adsk-green-300``, ``adsk-yellow-orange-700``, ``adsk-yellow-orange-500``, ``adsk-yellow-orange-300``,``adsk-dark-blue-700``, ``adsk-dark-blue-500``, ``adsk-dark-blue-300``, ``adsk-pink-700``,``adsk-pink-500``, ``adsk-pink-300``, ``adsk-turquoise-700``, ``adsk-turquoise-500``,``adsk-turquoise-300``, ``adsk-purple-700``, ``adsk-purple-500``, ``adsk-purple-300``,``adsk-salmon-700``, ``adsk-salmon-500``, ``adsk-salmon-300``, ``adsk-brown-700``, ``adsk-brown-500``,``adsk-brown-300``.Using colors other than those specified here is not disallowed, but may result in unexpected behavior inthe Assets UI.
    color: Optional[str] = None
    # A description of the status.
    description: Optional[str] = None
    # The display label of the status. This label appears in the Assets UI when viewing statuses. The label mustbe unique within this status set, but does not need to be unique within the project (in other words, thesame label can be used in multiple status sets in the same project). ``Uniqueness`` in this context meanscase-insensitive.
    label: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StatusStepSetsPostRequestBody_values:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StatusStepSetsPostRequestBody_values
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StatusStepSetsPostRequestBody_values()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "color": lambda n : setattr(self, 'color', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "label": lambda n : setattr(self, 'label', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("color", self.color)
        writer.write_str_value("description", self.description)
        writer.write_str_value("label", self.label)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithSubCostItemsPatchRequestBody(Parsable):
    # A user-defined identifier for the sub cost item, used for categorization or identification.Max length: 255
    code: Optional[str] = None
    # The recorded input quantity, typically used in performance tracking. For example, in labor tracking, ``inputQuantity`` represents man-hours utilized.
    input_quantity: Optional[float] = None
    # The field that is locked for cost calculations. This ensures the locked value remains unchanged during updates. Possible values: ``value``, ``quantity``, ``unitPrice``. For example, if you set ``unitPrice`` at $100, you can adjust the ``quantity`` while keeping the cost fixed.
    locked_field: Optional[str] = None
    # The name to give to the sub cost item.Max length: 1024
    name: Optional[str] = None
    # The planned number of units for the sub cost item.
    quantity: Optional[float] = None
    # The unit of measurement for the sub cost item. This value is configured in the ``Unit of measure`` settings for the project. Common units include ``ea`` (Each), ``gal`` (Gallon), and various volume, length, and time measurements.Max length: 1024
    unit: Optional[str] = None
    # The price per unit of the sub cost item.
    unit_price: Optional[float] = None
    # The total value of the sub cost item, calculated as ``quantity`` * ``unitPrice``.
    value: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithSubCostItemsPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithSubCostItemsPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithSubCostItemsPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "inputQuantity": lambda n : setattr(self, 'input_quantity', n.get_float_value()),
            "lockedField": lambda n : setattr(self, 'locked_field', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "unitPrice": lambda n : setattr(self, 'unit_price', n.get_float_value()),
            "value": lambda n : setattr(self, 'value', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("code", self.code)
        writer.write_float_value("inputQuantity", self.input_quantity)
        writer.write_str_value("lockedField", self.locked_field)
        writer.write_str_value("name", self.name)
        writer.write_float_value("quantity", self.quantity)
        writer.write_str_value("unit", self.unit)
        writer.write_float_value("unitPrice", self.unit_price)
        writer.write_float_value("value", self.value)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_rfi_patch_response_permitted_actions_remaining_reviewers import WithRfiPatchResponse_permittedActions_remainingReviewers
    from .with_rfi_patch_response_permitted_actions_update_rfi import WithRfiPatchResponse_permittedActions_updateRfi

@dataclass
class WithRfiPatchResponse_permittedActions(Parsable):
    # ``true``: The user can create a comment for the RFI.``false`` (default): The user cannot create a comment for the RFI.
    create_comment: Optional[bool] = None
    # ``true`` if the user can add a document reference to the RFI.``false`` if the user cannot add a document reference to the RFI.
    create_document_reference: Optional[bool] = None
    # ``true``: The user can create a response for the RFI.``false``: The user cannot create a response for the RFI.
    create_response: Optional[bool] = None
    # ``true``: The user can create a response on behalf of another user for the RFI.``false``: The user cannot create a response on behalf of another user for the RFI.
    create_response_on_behalf: Optional[bool] = None
    # Not relevant
    nudge: Optional[bool] = None
    # A list of users who are still expected to review the RFI.
    remaining_reviewers: Optional[list[WithRfiPatchResponse_permittedActions_remainingReviewers]] = None
    # ``true``: The user can remove a document reference to the RFI.``false``: The user cannot remove a document reference to the RFI.
    remove_document_reference: Optional[bool] = None
    # Not relevant
    share: Optional[bool] = None
    # The list of attributes and statuses available  for the user.
    update_rfi: Optional[WithRfiPatchResponse_permittedActions_updateRfi] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithRfiPatchResponse_permittedActions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithRfiPatchResponse_permittedActions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithRfiPatchResponse_permittedActions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_rfi_patch_response_permitted_actions_remaining_reviewers import WithRfiPatchResponse_permittedActions_remainingReviewers
        from .with_rfi_patch_response_permitted_actions_update_rfi import WithRfiPatchResponse_permittedActions_updateRfi

        from .with_rfi_patch_response_permitted_actions_remaining_reviewers import WithRfiPatchResponse_permittedActions_remainingReviewers
        from .with_rfi_patch_response_permitted_actions_update_rfi import WithRfiPatchResponse_permittedActions_updateRfi

        fields: dict[str, Callable[[Any], None]] = {
            "createComment": lambda n : setattr(self, 'create_comment', n.get_bool_value()),
            "createDocumentReference": lambda n : setattr(self, 'create_document_reference', n.get_bool_value()),
            "createResponse": lambda n : setattr(self, 'create_response', n.get_bool_value()),
            "createResponseOnBehalf": lambda n : setattr(self, 'create_response_on_behalf', n.get_bool_value()),
            "nudge": lambda n : setattr(self, 'nudge', n.get_bool_value()),
            "remainingReviewers": lambda n : setattr(self, 'remaining_reviewers', n.get_collection_of_object_values(WithRfiPatchResponse_permittedActions_remainingReviewers)),
            "removeDocumentReference": lambda n : setattr(self, 'remove_document_reference', n.get_bool_value()),
            "share": lambda n : setattr(self, 'share', n.get_bool_value()),
            "updateRfi": lambda n : setattr(self, 'update_rfi', n.get_object_value(WithRfiPatchResponse_permittedActions_updateRfi)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("createComment", self.create_comment)
        writer.write_bool_value("createDocumentReference", self.create_document_reference)
        writer.write_bool_value("createResponse", self.create_response)
        writer.write_bool_value("createResponseOnBehalf", self.create_response_on_behalf)
        writer.write_bool_value("nudge", self.nudge)
        writer.write_collection_of_object_values("remainingReviewers", self.remaining_reviewers)
        writer.write_bool_value("removeDocumentReference", self.remove_document_reference)
        writer.write_bool_value("share", self.share)
        writer.write_object_value("updateRfi", self.update_rfi)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .me_get_response_permitted_actions_create_rfi_permitted_statuses import MeGetResponse_permittedActions_createRfi_permittedStatuses

@dataclass
class MeGetResponse_permittedActions_createRfi(Parsable):
    """
    The user’s permissions for creating RFIs.Note that if this field is present, the user is permitted to create RFIs in the project.
    """
    # The list of statuses the user is permitted to transition an RFI to, without differentiating between workflow types (e.g., ``us`` and ``emea``).
    permitted_statuses: Optional[MeGetResponse_permittedActions_createRfi_permittedStatuses] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MeGetResponse_permittedActions_createRfi:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MeGetResponse_permittedActions_createRfi
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MeGetResponse_permittedActions_createRfi()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .me_get_response_permitted_actions_create_rfi_permitted_statuses import MeGetResponse_permittedActions_createRfi_permittedStatuses

        from .me_get_response_permitted_actions_create_rfi_permitted_statuses import MeGetResponse_permittedActions_createRfi_permittedStatuses

        fields: dict[str, Callable[[Any], None]] = {
            "permittedStatuses": lambda n : setattr(self, 'permitted_statuses', n.get_object_value(MeGetResponse_permittedActions_createRfi_permittedStatuses)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("permittedStatuses", self.permitted_statuses)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .get_filter_text_match_query_parameter_type import GetFilterTextMatchQueryParameterType
    from .item.with_user_item_request_builder import WithUserItemRequestBuilder
    from .users_get_response import UsersGetResponse
    from .users_post_request_body import UsersPostRequestBody
    from .users_post_response import UsersPostResponse

class UsersRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/admin/v1/projects/{projectId}/users
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new UsersRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/admin/v1/projects/{projectId}/users{?fields*,filterTextMatch*,filter%5BaccessLevels%5D*,filter%5BautodeskId%5D*,filter%5BcompanyId%5D*,filter%5BcompanyName%5D*,filter%5Bemail%5D*,filter%5Bid%5D*,filter%5Bname%5D*,filter%5Bproducts%5D*,filter%5BroleId%5D*,filter%5BroleIds%5D*,filter%5Bstatus%5D*,limit*,offset*,orFilters*,sort*}", path_parameters)
    
    def by_user_id(self,user_id: str) -> WithUserItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.admin.v1.projects.item.users.item collection
        param user_id: The ID of the user. To find the ID call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_. You can use either the ACC ID (``id``) or the Autodesk ID (``autodeskId``).
        Returns: WithUserItemRequestBuilder
        """
        if user_id is None:
            raise TypeError("user_id cannot be null.")
        from .item.with_user_item_request_builder import WithUserItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["userId"] = user_id
        return WithUserItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[UsersRequestBuilderGetQueryParameters]] = None) -> Optional[UsersGetResponse]:
        """
        Retrieves information about a filtered subset of users in the specified project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[UsersGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .users_get_response import UsersGetResponse

        return await self.request_adapter.send_async(request_info, UsersGetResponse, None)
    
    async def post(self,body: UsersPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[UsersPostResponse]:
        """
        Assigns a user to the specified project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[UsersPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .users_post_response import UsersPostResponse

        return await self.request_adapter.send_async(request_info, UsersPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[UsersRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves information about a filtered subset of users in the specified project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: UsersPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Assigns a user to the specified project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> UsersRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: UsersRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return UsersRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class UsersRequestBuilderGetQueryParameters():
        """
        Retrieves information about a filtered subset of users in the specified project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filteraccess_levels":
                return "filter%5BaccessLevels%5D"
            if original_name == "filterautodesk_id":
                return "filter%5BautodeskId%5D"
            if original_name == "filtercompany_id":
                return "filter%5BcompanyId%5D"
            if original_name == "filtercompany_name":
                return "filter%5BcompanyName%5D"
            if original_name == "filteremail":
                return "filter%5Bemail%5D"
            if original_name == "filterid":
                return "filter%5Bid%5D"
            if original_name == "filtername":
                return "filter%5Bname%5D"
            if original_name == "filterproducts":
                return "filter%5Bproducts%5D"
            if original_name == "filterrole_id":
                return "filter%5BroleId%5D"
            if original_name == "filterrole_ids":
                return "filter%5BroleIds%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "filter_text_match":
                return "filterTextMatch"
            if original_name == "or_filters":
                return "orFilters"
            if original_name == "fields":
                return "fields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # A list of the project fields to include in the response. Default value: all fields.Possible values: ``name``, ``email``, ``firstName``, ``lastName``, ``autodeskId``, ``analyticsId``, ``addressLine1``, ``addressLine2``, ``city``, ``stateOrProvince``, ``postalCode``, ``country``, ``imageUrl``, ``phone``, ``jobTitle``, ``industry``, ``aboutMe``, ``companyId``, ``accessLevels``, ``roleIds``, ``roles``, ``status``, ``addedOn`` and ``products``.
        fields: Optional[list[str]] = None

        # Specifies how text-based filters should match values in supported fields.This parameter can be used in any endpoint that supports text-based filtering (e.g., ``filter[name]``, ``filter[jobNumber]``, ``filter[companyName]``, etc.).Possible values:``contains`` (default) – Matches if the field contains the specified text anywhere``startsWith`` – Matches if the field starts with the specified text``endsWith`` – Matches if the field ends with the specified text``equals`` – Matches only if the field exactly matches the specified textMatching is case-insensitive.Wildcards and regular expressions are not supported.
        filter_text_match: Optional[GetFilterTextMatchQueryParameterType] = None

        # A list of user access levels that the returned users must have.Possible values: ``accouantAdmin``, ``projectAdmin``, ``executive``.Max length: 255
        filteraccess_levels: Optional[list[str]] = None

        # A list of the Autodesk IDs of users to retrieve.
        filterautodesk_id: Optional[list[str]] = None

        # The ID of a company that the returned users must represent.Max length: 255
        filtercompany_id: Optional[str] = None

        # The name of a company that returned users must be associated with. Can be a partial match based on the value of ``filterTextMatch`` that you provide.For example: ``filter[companyName]=Sample filterTextMatch=startsWith``Max length: 255
        filtercompany_name: Optional[str] = None

        # A user email address or address pattern that the returned users must have. This can be a partial match based on the value of ``filterTextMatch`` that you provide.For example:``filter[email]=sample filterTextMatch=startsWith``Max length: 255
        filteremail: Optional[str] = None

        # A list of the ACC IDs of users to retrieve.
        filterid: Optional[list[str]] = None

        # A user name or name pattern that the returned users must have. Can be a partial match based on the value of ``filterTextMatch`` that you provide.For example:``filter[name]=Sample filterTextMatch=startsWith``Max length: 255
        filtername: Optional[str] = None

        # A comma-separated list of the products that the returned users must have access to in the specified project. Only users that have access to one or more of the specified products are returned.Note that every product in the same account as the project is activated for the project, and a separate subset of these products is active for each user. This endpoint can retrieve users from ACC or BIM 360 projects.Some products are exclusive to ACC or to BIM 360, others are available on both platforms. Specify only the products on the appropriate platform for the users you want to retrieve.Possible ACC values: ``accountAdministration``, ``autoSpecs``, ``build``, ``buildingConnected``, ``capitalPlanning``, ``cloudWorksharing``, ``cost``, ``designCollaboration``, ``docs``, ``financials``, ``insight``, ``modelCoordination``, ``projectAdministration``, ``takeoff``, and ``workshopxr``.Possible BIM 360 values: ``accountAdministration``, ``assets``, ``cloudWorksharing``, ``costManagement``, ``designCollaboration``, ``documentManagement``, ``field``, ``fieldManagement``, ``glue``, ``insight``, ``modelCoordination``, ``plan``, ``projectAdministration``, ``projectHome``, ``projectManagement``, and ``quantification``.
        filterproducts: Optional[list[str]] = None

        # The ID of a user role that the returned users must have.To obtain a role ID for this filter, you can inspect the ``roleId`` field in previous responses to this endpoint or to the `GET projects/:projectId/users/:userId </en/docs/acc/v1/reference/http/admin-projects-projectId-users-userId-GET/>`_ endpoint.Max length: 255
        filterrole_id: Optional[UUID] = None

        # A list of the IDs of user roles that the returned users must have.To obtain a role ID for this filter, you can inspect the ``roleId`` field in previous responses to this endpoint or to the `GET projects/:projectId/users/:userId </en/docs/acc/v1/reference/http/admin-projects-projectId-users-userId-GET/>`_ endpoint.
        filterrole_ids: Optional[list[str]] = None

        # A list of statuses that the returned project users must be in. The default values are ``active`` and ``pending``.Possible values: ``active``, ``pending``, and ``deleted``.
        filterstatus: Optional[list[str]] = None

        # The maximum number of records to return in the response.Default: ``20``Minimum: ``1``Maximum: ``200`` (If a larger value is provided, only 200 records are returned)
        limit: Optional[int] = None

        # The index of the first record to return.Used for pagination in combination with the ``limit`` parameter.Example: ``limit=20`` and ``offset=40`` returns records 41–60.
        offset: Optional[int] = None

        # A list of user fields to combine with the SQL *OR* operator for filtering the returned project users. The *OR* is automatically incorporated between the fields; any one of them can produce a valid match.Possible values: ``id``, ``name``, ``email``, ``autodeskId``, ``status`` and ``accessLevels``.
        or_filters: Optional[list[str]] = None

        # A list of fields to sort the returned users by. Multiple sort fields are applied in sequence order - each sort field produces groupings of projects with the same values of that field; the next sort field applies within the groupings produced by the previous sort field.Each property can be followed by a direction modifier of either ``asc`` (ascending) or ``desc`` (descending). The default is ``asc``.Possible values: ``name``, ``email``, ``firstName``, ``lastName``, ``addressLine1``, ``addressLine2``, ``city``, ``companyName``, ``stateOrProvince``, ``status``, ``phone``, ``postalCode``, ``country`` and ``addedOn``. Default value: ``name``.
        sort: Optional[list[str]] = None

    
    @dataclass
    class UsersRequestBuilderGetRequestConfiguration(RequestConfiguration[UsersRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class UsersRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class GetFilterStatusQueryParameterType(str, Enum):
    FilterStatusINACTIVE = "filter[status]=INACTIVE",
    ACTIVE = "ACTIVE",
    INACTIVE = "INACTIVE",
    FilterInitiator = "filter[initiator]",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .values_batch_update_put_response_tabular_values_equipment_entries import ValuesBatchUpdatePutResponse_tabularValues_equipmentEntries
    from .values_batch_update_put_response_tabular_values_materials_entries import ValuesBatchUpdatePutResponse_tabularValues_materialsEntries
    from .values_batch_update_put_response_tabular_values_worklog_entries import ValuesBatchUpdatePutResponse_tabularValues_worklogEntries

@dataclass
class ValuesBatchUpdatePutResponse_tabularValues(Parsable):
    # Entries associated with equipment table in a Form.
    equipment_entries: Optional[list[ValuesBatchUpdatePutResponse_tabularValues_equipmentEntries]] = None
    # Entries associated with materials table in a Form.
    materials_entries: Optional[list[ValuesBatchUpdatePutResponse_tabularValues_materialsEntries]] = None
    # Entries associated with work log table in a Form.
    worklog_entries: Optional[list[ValuesBatchUpdatePutResponse_tabularValues_worklogEntries]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ValuesBatchUpdatePutResponse_tabularValues:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ValuesBatchUpdatePutResponse_tabularValues
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ValuesBatchUpdatePutResponse_tabularValues()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .values_batch_update_put_response_tabular_values_equipment_entries import ValuesBatchUpdatePutResponse_tabularValues_equipmentEntries
        from .values_batch_update_put_response_tabular_values_materials_entries import ValuesBatchUpdatePutResponse_tabularValues_materialsEntries
        from .values_batch_update_put_response_tabular_values_worklog_entries import ValuesBatchUpdatePutResponse_tabularValues_worklogEntries

        from .values_batch_update_put_response_tabular_values_equipment_entries import ValuesBatchUpdatePutResponse_tabularValues_equipmentEntries
        from .values_batch_update_put_response_tabular_values_materials_entries import ValuesBatchUpdatePutResponse_tabularValues_materialsEntries
        from .values_batch_update_put_response_tabular_values_worklog_entries import ValuesBatchUpdatePutResponse_tabularValues_worklogEntries

        fields: dict[str, Callable[[Any], None]] = {
            "equipmentEntries": lambda n : setattr(self, 'equipment_entries', n.get_collection_of_object_values(ValuesBatchUpdatePutResponse_tabularValues_equipmentEntries)),
            "materialsEntries": lambda n : setattr(self, 'materials_entries', n.get_collection_of_object_values(ValuesBatchUpdatePutResponse_tabularValues_materialsEntries)),
            "worklogEntries": lambda n : setattr(self, 'worklog_entries', n.get_collection_of_object_values(ValuesBatchUpdatePutResponse_tabularValues_worklogEntries)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("equipmentEntries", self.equipment_entries)
        writer.write_collection_of_object_values("materialsEntries", self.materials_entries)
        writer.write_collection_of_object_values("worklogEntries", self.worklog_entries)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .packages_get_response_results_version_type import PackagesGetResponse_results_versionType

@dataclass
class PackagesGetResponse_results(Parsable):
    # The time the package was created.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the package. For details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
    created_by: Optional[str] = None
    # The description of the package.Max length: 2048
    description: Optional[str] = None
    # The display ID of the package.
    display_id: Optional[int] = None
    # The unique identifier (UUID) of the package.
    id: Optional[UUID] = None
    # ``true``: The package is locked. Its contents cannot be modified until it is unlocked.``false``: The package is not locked. Files and resources can still be added, removed, or updated.
    locked: Optional[bool] = None
    # The time the package was locked.
    locked_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who locked the package. For details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
    locked_by: Optional[str] = None
    # The name of the package.Max length: 255
    name: Optional[str] = None
    # The number of resources in the package.
    resource_count: Optional[int] = None
    # The time the package was last updated.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the package. For details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
    updated_by: Optional[str] = None
    # The version type of the package.Possible values:- ``FIXED`` – The files in the package remain fixed at the selected versions.- ``CURRENT`` – The files in the package automatically update to the latest current versions.- ``CHANGING`` – The package is temporarily changing from one version type to another. This state usually lasts only a few seconds and cannot be used as a filter.For more details, see the `Change Package Version Type <https://help.autodesk.com/view/BUILD/ENU/?guid=View_Manage_Packages#change-package-version-type>`_ documentation.
    version_type: Optional[PackagesGetResponse_results_versionType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PackagesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PackagesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PackagesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .packages_get_response_results_version_type import PackagesGetResponse_results_versionType

        from .packages_get_response_results_version_type import PackagesGetResponse_results_versionType

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "displayId": lambda n : setattr(self, 'display_id', n.get_int_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "locked": lambda n : setattr(self, 'locked', n.get_bool_value()),
            "lockedAt": lambda n : setattr(self, 'locked_at', n.get_datetime_value()),
            "lockedBy": lambda n : setattr(self, 'locked_by', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "resourceCount": lambda n : setattr(self, 'resource_count', n.get_int_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "versionType": lambda n : setattr(self, 'version_type', n.get_object_value(PackagesGetResponse_results_versionType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("description", self.description)
        writer.write_int_value("displayId", self.display_id)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("locked", self.locked)
        writer.write_datetime_value("lockedAt", self.locked_at)
        writer.write_str_value("lockedBy", self.locked_by)
        writer.write_str_value("name", self.name)
        writer.write_int_value("resourceCount", self.resource_count)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_object_value("versionType", self.version_type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .queries_post_request_body_columns import QueriesPostRequestBody_columns
    from .queries_post_request_body_query import QueriesPostRequestBody_query

@dataclass
class QueriesPostRequestBody(Parsable):
    # `SQL AST for describing columns/projections </en/docs/acc/v1/tutorials/model-properties/query-ref/>`_
    columns: Optional[QueriesPostRequestBody_columns] = None
    # `SQL AST for binary expression/filter </en/docs/acc/v1/tutorials/model-properties/query-ref/>`_
    query: Optional[QueriesPostRequestBody_query] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> QueriesPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: QueriesPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return QueriesPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .queries_post_request_body_columns import QueriesPostRequestBody_columns
        from .queries_post_request_body_query import QueriesPostRequestBody_query

        from .queries_post_request_body_columns import QueriesPostRequestBody_columns
        from .queries_post_request_body_query import QueriesPostRequestBody_query

        fields: dict[str, Callable[[Any], None]] = {
            "columns": lambda n : setattr(self, 'columns', n.get_object_value(QueriesPostRequestBody_columns)),
            "query": lambda n : setattr(self, 'query', n.get_object_value(QueriesPostRequestBody_query)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("columns", self.columns)
        writer.write_object_value("query", self.query)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .folders.folders_request_builder import FoldersRequestBuilder
    from .naming_standards.naming_standards_request_builder import NamingStandardsRequestBuilder
    from .versions.versions_request_builder import VersionsRequestBuilder
    from .versions_batch_get.versions_batch_get_request_builder import VersionsBatchGetRequestBuilder

class ProjectItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/docs/v1/projects/{project-id}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ProjectItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/docs/v1/projects/{project%2Did}", path_parameters)
    
    @property
    def folders(self) -> FoldersRequestBuilder:
        """
        The folders property
        """
        from .folders.folders_request_builder import FoldersRequestBuilder

        return FoldersRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def naming_standards(self) -> NamingStandardsRequestBuilder:
        """
        The namingStandards property
        """
        from .naming_standards.naming_standards_request_builder import NamingStandardsRequestBuilder

        return NamingStandardsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def versions(self) -> VersionsRequestBuilder:
        """
        The versions property
        """
        from .versions.versions_request_builder import VersionsRequestBuilder

        return VersionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def versions_batch_get(self) -> VersionsBatchGetRequestBuilder:
        """
        The versionsBatchGet property
        """
        from .versions_batch_get.versions_batch_get_request_builder import VersionsBatchGetRequestBuilder

        return VersionsBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .assets.assets_request_builder import AssetsRequestBuilder
    from .assets_batch_create.assets_batch_create_request_builder import AssetsBatchCreateRequestBuilder
    from .assets_batch_delete.assets_batch_delete_request_builder import AssetsBatchDeleteRequestBuilder
    from .assets_batch_get.assets_batch_get_request_builder import AssetsBatchGetRequestBuilder
    from .assets_batch_patch.assets_batch_patch_request_builder import AssetsBatchPatchRequestBuilder

class WithProjectItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/assets/v2/projects/{projectId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithProjectItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/assets/v2/projects/{projectId}", path_parameters)
    
    @property
    def assets(self) -> AssetsRequestBuilder:
        """
        The assets property
        """
        from .assets.assets_request_builder import AssetsRequestBuilder

        return AssetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def assets_batch_create(self) -> AssetsBatchCreateRequestBuilder:
        """
        The assetsBatchCreate property
        """
        from .assets_batch_create.assets_batch_create_request_builder import AssetsBatchCreateRequestBuilder

        return AssetsBatchCreateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def assets_batch_delete(self) -> AssetsBatchDeleteRequestBuilder:
        """
        The assetsBatchDelete property
        """
        from .assets_batch_delete.assets_batch_delete_request_builder import AssetsBatchDeleteRequestBuilder

        return AssetsBatchDeleteRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def assets_batch_get(self) -> AssetsBatchGetRequestBuilder:
        """
        The assetsBatchGet property
        """
        from .assets_batch_get.assets_batch_get_request_builder import AssetsBatchGetRequestBuilder

        return AssetsBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def assets_batch_patch(self) -> AssetsBatchPatchRequestBuilder:
        """
        The assetsBatchPatch property
        """
        from .assets_batch_patch.assets_batch_patch_request_builder import AssetsBatchPatchRequestBuilder

        return AssetsBatchPatchRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .item.with_container_item_request_builder import WithContainerItemRequestBuilder

class ContainersRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/clash/v3/containers
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ContainersRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/clash/v3/containers", path_parameters)
    
    def by_container_id(self,container_id: UUID) -> WithContainerItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.bim360.clash.v3.containers.item collection
        param container_id: The GUID that uniquely identifies the container.
        Returns: WithContainerItemRequestBuilder
        """
        if container_id is None:
            raise TypeError("container_id cannot be null.")
        from .item.with_container_item_request_builder import WithContainerItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["containerId"] = container_id
        return WithContainerItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_form_patch_response_assignee_type import WithFormPatchResponse_assigneeType
    from .with_form_patch_response_custom_values import WithFormPatchResponse_customValues
    from .with_form_patch_response_form_template import WithFormPatchResponse_formTemplate
    from .with_form_patch_response_pdf_values import WithFormPatchResponse_pdfValues
    from .with_form_patch_response_status import WithFormPatchResponse_status
    from .with_form_patch_response_tabular_values import WithFormPatchResponse_tabularValues
    from .with_form_patch_response_weather import WithFormPatchResponse_weather

@dataclass
class WithFormPatchResponse(Parsable):
    # The unique identifier of the user, role, or company the form is assigned to.
    assignee_id: Optional[str] = None
    # Type of entity the form is assigned to.Possible values: ``company``, ``role``, ``user``
    assignee_type: Optional[WithFormPatchResponse_assigneeType] = None
    # When form was created on the server, UTC date and time in ISO-8601 format.
    created_at: Optional[datetime.datetime] = None
    # The unique identifier of the user who created the form.
    created_by: Optional[str] = None
    # For non-PDF forms, data stored in the form fields.
    custom_values: Optional[list[WithFormPatchResponse_customValues]] = None
    # Text for the form's description section.
    description: Optional[str] = None
    # Date the form pertains to.
    form_date: Optional[str] = None
    # A chronological user-fiendly identifier of the form within the project e.g. form #5.
    form_num: Optional[int] = None
    # Information about the form's template.
    form_template: Optional[WithFormPatchResponse_formTemplate] = None
    # The unique identifier of the form.
    id: Optional[str] = None
    # Unique identifier for the user that last re-opened the Form (if applicable).
    last_reopened_by: Optional[str] = None
    # Signature of the reviewer who last submitted the Form (if applicable). Signature value (base64 encoded SVG).
    last_submitter_signature: Optional[str] = None
    # Location identifier associated with the form. For more information about the location, see `GET nodes </en/docs/acc/v1/reference/http/locations-nodes-GET/>`_.
    location_id: Optional[str] = None
    # The name of the form instance.Max length: 100
    name: Optional[str] = None
    # Text for the form's notes section.
    notes: Optional[str] = None
    # For PDF forms, the URL to download the form's PDF.
    pdf_url: Optional[str] = None
    # For PDF forms, values extracted from fields in the PDF.
    pdf_values: Optional[list[WithFormPatchResponse_pdfValues]] = None
    # The unique identifier of the project the form belongs to.
    project_id: Optional[str] = None
    # The current status of the form. Note that forms are created in ``draft`` status. Possible values:``draft``: you can edit forms.``inReview``: you cannot edit forms, however, they can be approved by form reviewers.``submitted``: forms are closed and no longer editable.``archived`` forms are not edtiable and hidden in the UI.
    status: Optional[WithFormPatchResponse_status] = None
    # For non-PDF forms, data stored in the tables on the form.
    tabular_values: Optional[list[WithFormPatchResponse_tabularValues]] = None
    # When form was last updated, UTC date and time in ISO-8601 format.
    updated_at: Optional[datetime.datetime] = None
    # When form was created on the client, UTC date and time in ISO-8601 format.
    user_created_at: Optional[datetime.datetime] = None
    # Weather forecast captured on the form.
    weather: Optional[list[WithFormPatchResponse_weather]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithFormPatchResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithFormPatchResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithFormPatchResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_form_patch_response_assignee_type import WithFormPatchResponse_assigneeType
        from .with_form_patch_response_custom_values import WithFormPatchResponse_customValues
        from .with_form_patch_response_form_template import WithFormPatchResponse_formTemplate
        from .with_form_patch_response_pdf_values import WithFormPatchResponse_pdfValues
        from .with_form_patch_response_status import WithFormPatchResponse_status
        from .with_form_patch_response_tabular_values import WithFormPatchResponse_tabularValues
        from .with_form_patch_response_weather import WithFormPatchResponse_weather

        from .with_form_patch_response_assignee_type import WithFormPatchResponse_assigneeType
        from .with_form_patch_response_custom_values import WithFormPatchResponse_customValues
        from .with_form_patch_response_form_template import WithFormPatchResponse_formTemplate
        from .with_form_patch_response_pdf_values import WithFormPatchResponse_pdfValues
        from .with_form_patch_response_status import WithFormPatchResponse_status
        from .with_form_patch_response_tabular_values import WithFormPatchResponse_tabularValues
        from .with_form_patch_response_weather import WithFormPatchResponse_weather

        fields: dict[str, Callable[[Any], None]] = {
            "assigneeId": lambda n : setattr(self, 'assignee_id', n.get_str_value()),
            "assigneeType": lambda n : setattr(self, 'assignee_type', n.get_enum_value(WithFormPatchResponse_assigneeType)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "customValues": lambda n : setattr(self, 'custom_values', n.get_collection_of_object_values(WithFormPatchResponse_customValues)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "formDate": lambda n : setattr(self, 'form_date', n.get_str_value()),
            "formNum": lambda n : setattr(self, 'form_num', n.get_int_value()),
            "formTemplate": lambda n : setattr(self, 'form_template', n.get_object_value(WithFormPatchResponse_formTemplate)),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "lastReopenedBy": lambda n : setattr(self, 'last_reopened_by', n.get_str_value()),
            "lastSubmitterSignature": lambda n : setattr(self, 'last_submitter_signature', n.get_str_value()),
            "locationId": lambda n : setattr(self, 'location_id', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "pdfUrl": lambda n : setattr(self, 'pdf_url', n.get_str_value()),
            "pdfValues": lambda n : setattr(self, 'pdf_values', n.get_collection_of_object_values(WithFormPatchResponse_pdfValues)),
            "projectId": lambda n : setattr(self, 'project_id', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithFormPatchResponse_status)),
            "tabularValues": lambda n : setattr(self, 'tabular_values', n.get_collection_of_object_values(WithFormPatchResponse_tabularValues)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "userCreatedAt": lambda n : setattr(self, 'user_created_at', n.get_datetime_value()),
            "weather": lambda n : setattr(self, 'weather', n.get_collection_of_object_values(WithFormPatchResponse_weather)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("assigneeId", self.assignee_id)
        writer.write_enum_value("assigneeType", self.assignee_type)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_collection_of_object_values("customValues", self.custom_values)
        writer.write_str_value("description", self.description)
        writer.write_str_value("formDate", self.form_date)
        writer.write_int_value("formNum", self.form_num)
        writer.write_object_value("formTemplate", self.form_template)
        writer.write_str_value("id", self.id)
        writer.write_str_value("lastReopenedBy", self.last_reopened_by)
        writer.write_str_value("lastSubmitterSignature", self.last_submitter_signature)
        writer.write_str_value("locationId", self.location_id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("notes", self.notes)
        writer.write_str_value("pdfUrl", self.pdf_url)
        writer.write_collection_of_object_values("pdfValues", self.pdf_values)
        writer.write_str_value("projectId", self.project_id)
        writer.write_enum_value("status", self.status)
        writer.write_collection_of_object_values("tabularValues", self.tabular_values)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_datetime_value("userCreatedAt", self.user_created_at)
        writer.write_collection_of_object_values("weather", self.weather)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .takeoff_items_get_response_results_content_view import TakeoffItemsGetResponse_results_contentView
    from .takeoff_items_get_response_results_primary_quantity import TakeoffItemsGetResponse_results_primaryQuantity
    from .takeoff_items_get_response_results_property_values import TakeoffItemsGetResponse_results_propertyValues
    from .takeoff_items_get_response_results_secondary_quantities import TakeoffItemsGetResponse_results_secondaryQuantities
    from .takeoff_items_get_response_results_type import TakeoffItemsGetResponse_results_type

@dataclass
class TakeoffItemsGetResponse_results(Parsable):
    # The content view reference that this item is associated with.
    content_view: Optional[TakeoffItemsGetResponse_results_contentView] = None
    # The date and time when the resource was created, in the following format: ``YYYY-MM-DDThh:mm:ssZ``.
    created_at: Optional[datetime.datetime] = None
    # The geometry of a 2D takeoff item specified in SVG. This property is not returned for a 3D takeoff item.
    geometry: Optional[str] = None
    # The takeoff item ID.
    id: Optional[UUID] = None
    # The location ID associated with the takeoff item.For more information about the location, see `GET nodes </en/docs/acc/v1/reference/http/locations-nodes-GET/>`_.
    location_id: Optional[UUID] = None
    # The ID of the selected BIM model element in the viewing model.
    object_id: Optional[int] = None
    # The name of the takeoff type that the item is derived from.
    object_name: Optional[str] = None
    # The quantity of a takeoff.
    primary_quantity: Optional[TakeoffItemsGetResponse_results_primaryQuantity] = None
    # A list of takeoff item properties used to calculate quantity.
    property_values: Optional[list[TakeoffItemsGetResponse_results_propertyValues]] = None
    # A list of secondary takeoff quantities.
    secondary_quantities: Optional[list[TakeoffItemsGetResponse_results_secondaryQuantities]] = None
    # The takeoff type ID.
    takeoff_type_id: Optional[UUID] = None
    # The takeoff type method.Possible values: ``COUNT``, ``DISTANCE``, ``AREA``, ``SELECT``.Corresponding UI names: ``COUNT``, ``LINEAR``, ``AREA``, ``BIM``.
    type: Optional[TakeoffItemsGetResponse_results_type] = None
    # The date and time when the resource was last updated, in the following format: ``YYYY-MM-DDThh:mm:ssZ``.
    updated_at: Optional[datetime.datetime] = None
    # The name of the user who last updated the resource.
    updated_by_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TakeoffItemsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TakeoffItemsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TakeoffItemsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .takeoff_items_get_response_results_content_view import TakeoffItemsGetResponse_results_contentView
        from .takeoff_items_get_response_results_primary_quantity import TakeoffItemsGetResponse_results_primaryQuantity
        from .takeoff_items_get_response_results_property_values import TakeoffItemsGetResponse_results_propertyValues
        from .takeoff_items_get_response_results_secondary_quantities import TakeoffItemsGetResponse_results_secondaryQuantities
        from .takeoff_items_get_response_results_type import TakeoffItemsGetResponse_results_type

        from .takeoff_items_get_response_results_content_view import TakeoffItemsGetResponse_results_contentView
        from .takeoff_items_get_response_results_primary_quantity import TakeoffItemsGetResponse_results_primaryQuantity
        from .takeoff_items_get_response_results_property_values import TakeoffItemsGetResponse_results_propertyValues
        from .takeoff_items_get_response_results_secondary_quantities import TakeoffItemsGetResponse_results_secondaryQuantities
        from .takeoff_items_get_response_results_type import TakeoffItemsGetResponse_results_type

        fields: dict[str, Callable[[Any], None]] = {
            "contentView": lambda n : setattr(self, 'content_view', n.get_object_value(TakeoffItemsGetResponse_results_contentView)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "geometry": lambda n : setattr(self, 'geometry', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "locationId": lambda n : setattr(self, 'location_id', n.get_uuid_value()),
            "objectId": lambda n : setattr(self, 'object_id', n.get_int_value()),
            "objectName": lambda n : setattr(self, 'object_name', n.get_str_value()),
            "primaryQuantity": lambda n : setattr(self, 'primary_quantity', n.get_object_value(TakeoffItemsGetResponse_results_primaryQuantity)),
            "propertyValues": lambda n : setattr(self, 'property_values', n.get_collection_of_object_values(TakeoffItemsGetResponse_results_propertyValues)),
            "secondaryQuantities": lambda n : setattr(self, 'secondary_quantities', n.get_collection_of_object_values(TakeoffItemsGetResponse_results_secondaryQuantities)),
            "takeoffTypeId": lambda n : setattr(self, 'takeoff_type_id', n.get_uuid_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(TakeoffItemsGetResponse_results_type)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("contentView", self.content_view)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("geometry", self.geometry)
        writer.write_uuid_value("id", self.id)
        writer.write_uuid_value("locationId", self.location_id)
        writer.write_int_value("objectId", self.object_id)
        writer.write_str_value("objectName", self.object_name)
        writer.write_object_value("primaryQuantity", self.primary_quantity)
        writer.write_collection_of_object_values("propertyValues", self.property_values)
        writer.write_collection_of_object_values("secondaryQuantities", self.secondary_quantities)
        writer.write_uuid_value("takeoffTypeId", self.takeoff_type_id)
        writer.write_enum_value("type", self.type)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedByName", self.updated_by_name)
    




from enum import Enum

class ValuesBatchUpdatePutRequestBody_tabularValues_columns_columnName(str, Enum):
    Trade = "trade",
    TextVal = "textVal",
    Timespan = "timespan",
    TimespanVal = "timespanVal",
    Headcount = "headcount",
    NumberVal = "numberVal",
    Description = "description",
    Item = "item",
    Quantity = "quantity",
    Unit = "unit",

